@@ -4,21 +4,7 @@ import struct
 from datasketch.storage import (
     ordered_storage, unordered_storage, _random_name)
 
-_integration_precision = 0.001
-def _integration(f, a, b):
-    p = _integration_precision
-    area = 0.0
-    x = a
-    while x < b:
-        area += f(x+0.5*p)*p
-        x += p
-    return area, None
-
-try:
-    from scipy.integrate import quad as integrate
-except ImportError:
-    # For when no scipy installed
-    integrate = _integration
+from scipy.integrate import quad as integrate
 
 
 def _false_positive_probability(threshold, b, r):
@@ -121,7 +107,7 @@ class MinHashLSH(object):
                     false_positive_weight, false_negative_weight)
 
         self.prepickle = storage_config['type'] == 'redis' if prepickle is None else prepickle
-        
+
         self.hashfunc = hashfunc
         if hashfunc:
             self._H = self._hashed_byteswap
@@ -211,6 +197,50 @@ class MinHashLSH(object):
         else:
             return list(candidates)
 
+    def add_to_query_buffer(self, minhash):
+        '''
+        Giving the MinHash of the query set, buffer
+        queries to retrieve the keys that references
+        sets with Jaccard similarities greater than
+        the threshold.
+
+        Buffered queries can be executed using
+        `collect_query_buffer`. The combination of these
+        functions is way faster if cassandra backend
+        is used with `shared_buffer`.
+
+        Args:
+            minhash (datasketch.MinHash): The MinHash of the query set.
+        '''
+        if len(minhash) != self.h:
+            raise ValueError("Expecting minhash with length %d, got %d"
+                             % (self.h, len(minhash)))
+        for (start, end), hashtable in zip(self.hashranges, self.hashtables):
+            H = self._H(minhash.hashvalues[start:end])
+            hashtable.add_to_select_buffer([H])
+
+    def collect_query_buffer(self):
+        '''
+        Execute and return buffered queries given
+        by `add_to_query_buffer`.
+
+        If multiple query MinHash were added to the query buffer,
+        the intersection of the results of all query MinHash will be returned.
+
+        Returns:
+            `list` of unique keys.
+        '''
+        collected_result_sets = [
+            set(collected_result_lists)
+            for hashtable in self.hashtables
+            for collected_result_lists in hashtable.collect_select_buffer()
+        ]
+        if not collected_result_sets:
+            return []
+        if self.prepickle:
+            return [pickle.loads(key) for key in set.intersection(*collected_result_sets)]
+        return list(set.intersection(*collected_result_sets))
+
     def __contains__(self, key):
         '''
         Args:
@@ -253,7 +283,7 @@ class MinHashLSH(object):
 
     def _hashed_byteswap(self, hs):
         return self.hashfunc(bytes(hs.byteswap().data))
-    
+
     def _query_b(self, minhash, b):
         if len(minhash) != self.h:
             raise ValueError("Expecting minhash with length %d, got %d"
